import { Belt, getTooBelts, lockBelts, setupContainer, unlockBelts } from "./libraries/Belt";
import { system, world } from "@minecraft/server";
import { LightLevel } from "./libraries/utils";
import DynamicLight from "./modules/DynamicLight";
const Belts = new Map();
world.gameRules.showTags = false;
system.afterEvents.scriptEventReceive.subscribe(({ sourceEntity: player }) => {
    if (!player?.isPlayer() || !player.isValid())
        return;
    player.setDynamicProperty('has_totem', system.currentTick);
    CheckLight(player);
    const { dimension, id: playerID } = player;
    const slot = player.inventory.container.getSlot(player.selectedSlotIndex);
    const isBelt = slot.hasItem() && slot.typeId === 'rts:tool_belt';
    if (!isBelt && !Belts.has(playerID))
        return;
    const old = Belts.get(playerID);
    const belt = old?.belt ?? new Belt(slot);
    const entity = dimension.getEntities({ type: 'rts:belt_container', tags: [`rts:${playerID}`] })[0];
    if (!isBelt || (entity && entity.getDynamicProperty('ID') !== slot.getDynamicProperty('ID'))) {
        if (!entity?.isValid())
            return;
        belt.saveInventory(entity, old?.slot);
        Belts.delete(playerID);
        unlockBelts(player);
        return entity.remove();
    }
    lockBelts(player);
    const container = entity ?? setupContainer(player, belt, slot.nameTag);
    container.teleport(player.getHeadLocation(), { dimension });
    Belts.set(playerID, { belt, slot });
});
function CheckLight(player) {
    const belts = getTooBelts(player);
    let level = 0;
    for (const belt of belts) {
        const { data } = belt;
        for (let i = 0; i < data.length; i++) {
            const { typeId } = data[i];
            const light = LightLevel[typeId];
            if (!light)
                continue;
            if (light > level)
                level = light;
        }
    }
    DynamicLight(player, level);
}
import "./modules/DynamicLight";
import "./modules/Blocks";
import "./modules/Tools";
