import { BlockPermutation, BlockTypes, system, world } from "@minecraft/server";
import { sleep, Vector } from "../libraries/utils";
const isOldVersion = !BlockTypes.get('minecraft:light_block_0');
export default function DynamicLight(player, level) {
    const getHead = () => player.getHeadLocation();
    SetLight(player, getHead, level);
    const reduced = Math.max(level - 2, 1);
    SetLight(player, () => Vector.add(getHead(), { y: 1 }), reduced);
    SetLight(player, () => Vector.add(getHead(), { x: 1 }), reduced);
    SetLight(player, () => Vector.add(getHead(), { x: -1 }), reduced);
    SetLight(player, () => Vector.add(getHead(), { z: 1 }), reduced);
    SetLight(player, () => Vector.add(getHead(), { z: -1 }), reduced);
}
export async function SetLight(player, getPos, level = 15) {
    const { dimension } = player;
    const location = getPos();
    const block = dimension.getBlock(location);
    if (!block?.isAir && !block?.typeId.match(/:light_block/))
        return;
    const level_ = isOldVersion ? block.typeId.match(/:light_block_([0-9]+)/)?.[1] : block.permutation.getState('block_light_level');
    if (Number(level_) >= level)
        return;
    const light = lightBlock(level);
    const old = block.permutation;
    const ID = `DL:${dimension.id}:${JSON.stringify(location)}`;
    world.setDynamicProperty(ID, JSON.stringify({ location, dimID: dimension.id, }));
    block.setPermutation(light);
    async function run() {
        await sleep();
        if (Vector.equals(getPos(), location))
            return run();
        try {
            if (block?.matches(light.type.id, light.getAllStates()))
                block.setPermutation(old);
        }
        catch { }
        world.setDynamicProperty(ID);
    }
    run();
}
world.getDynamicPropertyIds().forEach(function RemoveLight(ID) {
    if (!ID.startsWith('DL:'))
        return;
    const { location, dimID } = JSON.parse(world.getDynamicProperty(ID));
    try {
        const block = world.getDimension(dimID).getBlock(location);
        if (block.typeId.match(/:light_block/))
            block.setType('air');
        world.setDynamicProperty(ID);
    }
    catch {
        system.run(() => RemoveLight(ID));
    }
});
function lightBlock(level) {
    return isOldVersion ?
        BlockPermutation.resolve('minecraft:light_block', { block_light_level: level })
        : BlockPermutation.resolve('minecraft:light_block_' + level);
}
