import { getTooBelts, getToolDefinition, parseItem, setupContainer, validateDefinition } from "../libraries/Belt";
import { EquipmentSlot, ItemStack, system } from "@minecraft/server";
import { afterEvents, beforeEvents, sleep, Vector } from "../libraries/utils";
const cooldown = new Map();
afterEvents.entityHitBlock.subscribe(async ({ hitBlock: block, hitBlockPermutation: permutation, damagingEntity: player }) => {
    if (!player.isPlayer() || player.isSneaking || block.isAir || block.isLiquid || cooldown.has(player.id))
        return;
    const equippable = player.equippable;
    const hand = equippable.getEquipment(EquipmentSlot.Mainhand);
    if (!hand)
        return;
    const definition = getToolDefinition(hand);
    if (!definition || validateDefinition(definition, permutation))
        return;
    let default_;
    const belts = getTooBelts(player);
    for (const belt of belts) {
        const data = belt.data;
        for (let j = 0; j < data.length; j++) {
            const item = new ItemStack(data[j].typeId);
            if (!item)
                continue;
            const definition = getToolDefinition(item);
            if (!default_ && definition?.default)
                default_ = { belt, slot: j };
            if (!definition || !validateDefinition(definition, permutation))
                continue;
            await sleep();
            const entity = setupContainer(player, belt, '', Vector.multiply(player.getViewDirection(), -1));
            belt.loadInventory(entity);
            player.inventory.container?.swapItems(player.selectedSlotIndex, data[j].slot, entity.inventory?.container);
            player.dimension.playSound('armor.equip_generic', player.getHeadLocation());
            belt.saveInventory(entity);
            return entity.remove();
        }
    }
    if (!default_)
        return;
    const belt = default_.belt;
    const entity = setupContainer(player, belt, '', Vector.multiply(player.getViewDirection(), -1));
    belt.loadInventory(entity);
    player.inventory.container?.swapItems(player.selectedSlotIndex, default_.slot, entity.inventory?.container);
    player.dimension.playSound('armor.equip_generic', player.getHeadLocation());
    belt.saveInventory(entity);
    entity.remove();
});
const beforeCooldown = new Map();
beforeEvents.itemUseOn.subscribe(async ({ block, source: player, itemStack: hand }) => {
    if (beforeCooldown.has(player.id) || player.isSneaking)
        return;
    beforeCooldown.set(player.id, system.runTimeout(() => beforeCooldown.delete(player.id), 5));
    const definition = getToolDefinition(hand);
    if (!definition || validateDefinition(definition, block.permutation, true))
        return;
    const belts = getTooBelts(player);
    for (const belt of belts) {
        const { data } = belt;
        for (let j = 0; j < data.length; j++) {
            const item = new ItemStack(data[j].typeId);
            const definition = getToolDefinition(item);
            if (!definition || !validateDefinition(definition, block.permutation, true))
                continue;
            await sleep();
            const entity = setupContainer(player, belt, '', Vector.multiply(player.getViewDirection(), -1));
            belt.loadInventory(entity);
            player.inventory.container?.swapItems(player.selectedSlotIndex, data[j].slot, entity.inventory?.container);
            player.dimension.playSound('armor.equip_generic', player.getHeadLocation());
            belt.saveInventory(entity);
            return entity.remove();
        }
    }
});
afterEvents.playerBreakBlock.subscribe(async ({ itemStackBeforeBreak: before, itemStackAfterBreak: after, player, brokenBlockPermutation: permutation }) => {
    if (!before || after)
        return;
    const definition = getToolDefinition(before);
    if (!definition)
        return;
    let default_;
    const belts = getTooBelts(player);
    for (const belt of belts) {
        const data = belt.data;
        for (let j = 0; j < data.length; j++) {
            const item = parseItem(data[j]);
            if (!item)
                continue;
            const definition = getToolDefinition(item);
            if (!default_ && definition?.default)
                default_ = { belt, slot: j };
            if (!definition || !validateDefinition(definition, permutation))
                continue;
            await sleep();
            const entity = setupContainer(player, belt, '', Vector.multiply(player.getViewDirection(), -1));
            belt.loadInventory(entity);
            player.inventory.container?.swapItems(player.selectedSlotIndex, data[j].slot, entity.inventory?.container);
            player.dimension.playSound('armor.equip_generic', player.getHeadLocation());
            belt.saveInventory(entity);
            return entity.remove();
        }
    }
});
afterEvents.entityHitEntity.subscribe(async ({ damagingEntity: player, hitEntity: target }) => {
    if (!player.isPlayer() || player.isSneaking)
        return;
    const family = target.getComponent('type_family');
    if (!family || family.hasTypeFamily('inanimate'))
        return;
    cooldown.set(player.id, system.runTimeout(() => cooldown.delete(player.id), 20 * 2));
    const equippable = player.equippable;
    const hand = equippable.getEquipment(EquipmentSlot.Mainhand);
    if (!hand)
        return;
    const definition = getToolDefinition(hand);
    if (!definition || definition.weapon)
        return;
    const belts = getTooBelts(player);
    const weapons = [];
    for (const belt of belts) {
        const data = belt.data;
        for (let j = 0; j < data.length; j++) {
            const item = parseItem(data[j]);
            if (!item)
                continue;
            const definition = getToolDefinition(item);
            if (definition?.weapon)
                weapons.push({ belt, slot: data[j].slot, priority: definition.weapon });
        }
    }
    if (!weapons.length)
        return;
    const { belt, slot } = weapons.reduce((a, b) => a.priority < b.priority ? a : b);
    await sleep();
    const entity = setupContainer(player, belt, '', Vector.multiply(player.getViewDirection(), -1));
    belt.loadInventory(entity);
    player.inventory.container?.swapItems(player.selectedSlotIndex, slot, entity.inventory?.container);
    player.dimension.playSound('armor.equip_generic', player.getHeadLocation());
    belt.saveInventory(entity);
    return entity.remove();
});
afterEvents.entityHurt.subscribe(({ hurtEntity: player }) => {
    if (!player.isPlayer())
        return;
    const tick = system.currentTick;
    const last = player.getDynamicProperty('has_totem') ?? tick;
    if (tick - last > 5)
        return;
    const equippable = player.equippable;
    const offhand = equippable.getEquipmentSlot(EquipmentSlot.Offhand);
    if (offhand?.hasItem())
        return;
    const belts = getTooBelts(player);
    for (const belt of belts) {
        const data = belt.data;
        for (let j = 0; j < data.length; j++) {
            if (data[j].typeId !== 'minecraft:totem_of_undying')
                continue;
            const entity = setupContainer(player, belt, '', Vector.multiply(player.getViewDirection(), -1));
            belt.loadInventory(entity);
            const inv = entity.inventory?.container;
            const item = inv.getItem(data[j].slot);
            inv.setItem(data[j].slot);
            belt.saveInventory(entity);
            player.dimension.playSound('armor.equip_generic', player.getHeadLocation());
            offhand.setItem(item);
            return entity.remove();
        }
    }
});
